@@ -6,21 +6,21 @@
 #' @templateVar stanregArg object
 #' @template args-stanreg-object
 #' @param re.form For models with group-level terms, \code{re.form} is
-#'   passed to \code{\link{posterior_linpred}} if specified.
+#'   passed to \code{\link{posterior_epred}} if specified.
 #' @param ... Currently ignored.
 #' 
 #' @return A vector of R-squared values with length equal to the posterior
 #'   sample size (the posterior distribution of R-squared).
 #'   
 #' @references
-#' Andrew Gelman, Ben Goodrich, Jonah Gabry, and Aki Vehtari (2018). R-squared
+#' Andrew Gelman, Ben Goodrich, Jonah Gabry, and Aki Vehtari (2019). R-squared
 #' for Bayesian regression models. \emph{The American Statistician}, to appear.
-#' DOI: 10.1080/00031305.2018.1549100.
-#' (\href{https://doi.org/10.1080/00031305.2018.1549100}{Journal},
-#' \href{http://www.stat.columbia.edu/~gelman/research/published/bayes_R2_v3.pdf}{Preprint},
+#' \doi{10.1080/00031305.2018.1549100}
+#' (\href{https://www.tandfonline.com/doi/abs/10.1080/00031305.2018.1549100}{Article},
 #' \href{https://avehtari.github.io/bayes_R2/bayes_R2.html}{Notebook})
 #' 
 #' @examples
+#' if (.Platform$OS.type != "windows" || .Platform$r_arch != "i386") {
 #' fit <- stan_glm(
 #'   mpg ~ wt + cyl, 
 #'   data = mtcars, 
@@ -40,7 +40,7 @@
 #' print(example_model)
 #' median(bayes_R2(example_model))
 #' median(bayes_R2(example_model, re.form = NA)) # exclude group-level
-#' 
+#' }
 bayes_R2.stanreg <- function(object, ..., re.form = NULL) {
     
     if (!used.sampling(object))
@@ -53,7 +53,7 @@ bayes_R2.stanreg <- function(object, ..., re.form = NULL) {
       stop("bayes_R2 is only available for Gaussian and binomial models.")
     }
     
-    mu_pred <- posterior_linpred(object, transform = TRUE, re.form = re.form)
+    mu_pred <- posterior_epred(object, re.form = re.form)
     if (is.binomial(fam)) {
       y <- get_y(object)
       if (NCOL(y) == 2) {
@@ -99,7 +99,7 @@ loo_R2.stanreg <- function(object, ...) {
     psis_object <- loo::psis(log_ratios, r_eff = NA)
   }
   
-  mu_pred <- posterior_linpred(object, transform = TRUE)
+  mu_pred <- posterior_epred(object)
   if (is.binomial(fam)) {
     if (is.factor(y)) {
       y <- fac2bin(y)
