@@ -17,15 +17,13 @@
 
 #' Applied Regression Modeling via RStan
 #'
-#' @docType package
-#' @name rstanarm-package
-#' @aliases rstanarm
 #' @useDynLib rstanarm, .registration = TRUE
 #'
 #' @import methods
 #' @importFrom rstan optimizing sampling vb constrain_pars extract
 #'   extract_sparse_parts get_posterior_mean stanc
 #' @importFrom utils capture.output
+#' @importFrom RcppParallel RcppParallelLibs
 #' @import stats
 #' @import Rcpp
 #' @import bayesplot
@@ -93,8 +91,8 @@
 #'   question about \pkg{rstanarm} on the Stan-users forum.
 #' }
 #'
-#' @templateVar armRef \url{https://stat.columbia.edu/~gelman/arm/}
-#' @templateVar bdaRef \url{https://stat.columbia.edu/~gelman/book/}
+#' @templateVar armRef \url{https://sites.stat.columbia.edu/gelman/arm/}
+#' @templateVar bdaRef \url{https://sites.stat.columbia.edu/gelman/book/}
 #' @template reference-lme4
 #' @template reference-bda
 #' @template reference-gelman-hill
@@ -103,4 +101,4 @@
 #' @template reference-bayesvis
 #' @template reference-muth
 #'
-NULL
+"_PACKAGE"
