@@ -148,13 +148,13 @@ reformulate_lhs <- function(x) {
 # Reformulate an expression as the RHS of a model formula
 # 
 # @param x The expression to reformulate
-# @param subbars A logical specifying whether to call lme4::subbars
+# @param subbars A logical specifying whether to call reformulas::subbars
 #   on the result
 # @return A model formula
 reformulate_rhs <- function(x, subbars = FALSE) {
   fm <- formula(substitute(~ RHS, list(RHS = x)))
   if (subbars) {
-    lme4::subbars(fm)
+    reformulas::subbars(fm)
   } else {
     fm
   }
@@ -268,10 +268,10 @@ split_cov_prior <- function(prior_stuff, cnms, submodel_cnms) {
 # @return A named list with the same structure as returned by handle_glm_prior
 autoscale_prior <- function(prior_stuff, response = NULL, predictors = NULL, 
                             family = NULL, QR = FALSE, min_prior_scale = 1e-12, 
-                            assoc = NULL, ...) {
+                            assoc = NULL, scale_assoc = NULL, ...) {
   ps <- prior_stuff
   
-  if (!is.null(response) && is.gaussian(family)) { 
+  if (!identical(NULL, response) && is.gaussian(family$family)) { 
     # use response variable for scaling priors
     if (ps$prior_dist > 0L && ps$prior_autoscale) {
       ss <- sd(response)
@@ -279,7 +279,7 @@ autoscale_prior <- function(prior_stuff, response = NULL, predictors = NULL,
     }
   }
   
-  if (!is.null(predictors) && !QR) {
+  if (!identical(NULL, predictors) && !QR) {
     # use predictors for scaling priors
     if (ps$prior_dist > 0L && ps$prior_autoscale) {
       ps$prior_scale <- 
@@ -288,18 +288,20 @@ autoscale_prior <- function(prior_stuff, response = NULL, predictors = NULL,
     }      
   }
   
-  if (!is.null(assoc)) {
+  if (!identical(NULL, assoc)) {
     # Evaluate mean and SD of each of the association terms that will go into
     # the linear predictor for the event submodel (as implicit "covariates").
     # (NB the approximate association terms are calculated using coefs
     # from the separate longitudinal submodels estimated using glmer).
     # The mean will be used for centering each association term.
     # The SD will be used for autoscaling the prior for each association parameter.
-    if (is.null(family))
+    if (identical(NULL, family))
       stop("'family' cannot be NULL when autoscaling association parameters.")
     assoc_terms <- make_assoc_terms(family = family, assoc = assoc, ...)
     ps$a_xbar <- as.array(apply(assoc_terms, 2L, mean))
     if (ps$prior_dist > 0L && ps$prior_autoscale) {
+      if (!identical(NULL, scale_assoc))
+        assoc_terms <- assoc_terms * scale_assoc
       a_beta_scale <- apply(assoc_terms, 2L, get_scale_value)
       ps$prior_scale <- pmax(min_prior_scale, ps$prior_scale / a_beta_scale)
     }
@@ -577,7 +579,7 @@ rename_t_and_cauchy <- function(prior_stuff, has) {
 #   has_aux: logical specifying whether the glmer submodel 
 #     requires an auxiliary parameter.
 handle_y_mod <- function(formula, data, family) {
-  mf <- stats::model.frame(lme4::subbars(formula), data)
+  mf <- stats::model.frame(reformulas::subbars(formula), data)
   if (!length(formula) == 3L)
     stop2("An outcome variable must be specified.")
   
@@ -605,8 +607,12 @@ handle_y_mod <- function(formula, data, family) {
   has_aux <- check_for_aux(family)
   family <- append_mvmer_famlink(family, is_bernoulli)
   
+  # Offset
+  offset <- model.offset(mf)
+  has_offset <- as.numeric(!is.null(offset))
+  
   nlist(y, x, z, reTrms, model_frame = mf, formula, terms, 
-        family, intercept_type, has_aux)
+        family, intercept_type, has_aux, offset, has_offset)
 }
 
 # Return the response vector for passing to Stan
@@ -641,7 +647,7 @@ make_y_for_stan <- function(formula, model_frame, family) {
 #   N,K: number of rows (observations) and columns (predictors) in the
 #     fixed effects model matrix
 make_x_for_stan <- function(formula, model_frame) {
-  x_form <- lme4::nobars(formula)
+  x_form <- reformulas::nobars(formula)
   x <- model.matrix(x_form, model_frame)
   has_intercept <- check_for_intercept(x, logical = TRUE)
   xtemp <- drop_intercept(x)
@@ -674,7 +680,7 @@ make_x_for_stan <- function(formula, model_frame) {
 #     grouping factor
 #   ngrps: a vector with the number of groups for each grouping factor 
 make_z_for_stan <- function(formula, model_frame) {
-  bars <- lme4::findbars(formula)
+  bars <- reformulas::findbars(formula)
   if (length(bars) > 2L)
     stop2("A maximum of 2 grouping factors are allowed.")
   z_parts <- lapply(bars, split_at_bars)
@@ -810,7 +816,7 @@ append_mvmer_famlink <- function(family, is_bernoulli = FALSE) {
 #   - the formula part (ie. the formula on the LHS of "|"), and 
 #   - the name of the grouping factor (ie. the variable on the RHS of "|")
 #
-# @param x Random effects part of a model formula, as returned by lme4::findbars
+# @param x Random effects part of a model formula, as returned by reformulas::findbars
 # @return A named list with the following elements:
 #   re_form: a formula specifying the random effects structure
 #   group_var: the name of the grouping factor
@@ -913,13 +919,13 @@ check_id_list <- function(id_var, y_flist) {
 #
 # @param terms The existing model frame terms object
 # @param formula The formula that was used to build the model frame
-#   (but prior to having called lme4::subbars on it!)
+#   (but prior to having called reformulas::subbars on it!)
 # @param data The data frame that was used to build the model frame
 # @return A terms object with predvars.fixed and predvars.random as
 #   additional attributes
 append_predvars_attribute <- function(terms, formula, data) {
-  fe_form <- lme4::nobars(formula)
-  re_form <- lme4::subbars(justRE(formula, response = TRUE))
+  fe_form <- reformulas::nobars(formula)
+  re_form <- reformulas::subbars(justRE(formula, response = TRUE))
   fe_frame <- stats::model.frame(fe_form, data)
   re_frame <- stats::model.frame(re_form, data)
   fe_terms <- attr(fe_frame, "terms")
@@ -956,7 +962,7 @@ use_predvars <- function(mod, keep_response = TRUE) {
   if (keep_response && length(fm) == 3L) {
     fm <- reformulate(rhs, response = formula(mod)[[2L]])
   } else if (keep_response && length(fm) == 2L) {
-    warning2("No response variable found, reformulating RHS only.")
+    warning("No response variable found, reformulating RHS only.", call. = FALSE)
     fm <- reformulate(rhs, response = NULL)
   } else {
     fm <- reformulate(rhs, response = NULL)
@@ -1490,6 +1496,56 @@ validate_lag_assoc <- function(lag_assoc, M) {
   lag_assoc
 }
 
+# Validate the user input to the scale_assoc argument of stan_jm
+#
+# @param scale_assoc The user input to the scale_assoc argument
+# @param assoc_as_list A list with information about the association structure for 
+#   the longitudinal submodels
+# @return A numeric vector of scaling parameters for all assoc terms
+validate_scale_assoc <- function(scale_assoc, assoc_as_list) {
+  M <- length(assoc_as_list)
+  
+  if (is.null(scale_assoc)) 
+    scale_assoc <-  rep(1,M)
+  if (length(scale_assoc) < M)
+    stop2("'scale_assoc' must be specified for each longitudinal submodel.")
+  if (length(scale_assoc) > M)
+    stop2("'scale_assoc' can only be specified once for each longitudinal submodel.")
+  if (!is.numeric(scale_assoc))
+    stop2("'scale_assoc' must be numeric.")
+  
+  sel_shared <- c("shared_b", "shared_coef")
+  sel_terms <- c("etavalue", "etaslope", "etaauc", 
+                 "muvalue", "muslope", "muauc")
+  sel_data <- c("which_formulas")
+  sel_itx <- c("which_interactions")
+  
+  scale_list <- list()
+  for (m in 1:M) {
+    a = assoc_as_list[[m]]
+    
+    if (a[["null"]]) {
+      scale_list[[m]] = as.array(integer(0))
+    } else {
+      if (scale_assoc[m] == 0)
+        stop2("'scale_assoc' must be non-zero.")
+      if (any(unlist(a[sel_shared])))
+        stop2("'scale_assoc' is not yet implemented for the following association structures: ", 
+              paste(sel_shared, collapse = ", "))
+      
+      # calculate scale for each assoc term
+      scale_terms <- rep(scale_assoc[m], length(which(unlist(a[sel_terms]))))
+      scale_data <- rep(scale_assoc[m], length(unlist(a[[sel_data]])))
+      scale_itx <- scale_assoc[m] * scale_assoc[unlist(a[[sel_itx]])]
+      
+      scale_list[[m]] <- c(scale_terms, scale_data, scale_itx)
+    }
+  }
+  
+  # return vector of scaling parameters
+  return(unlist(scale_list))
+}
+
 # Remove suffixes from the user inputted assoc argument
 #
 # @param x A character vector, being the user input to the 
@@ -1531,7 +1587,7 @@ parse_assoc_data <- function(x, user_x) {
     if (identical(length(fm), 3L))
       stop(paste0("Formula specified for '", x, "' association structure should not ",
                   "include a response."), call. = FALSE)
-    if (length(lme4::findbars(fm)))
+    if (length(reformulas::findbars(fm)))
       stop(paste0("Formula specified for '", x, "' association structure should only ",
                   "include fixed effects."), call. = FALSE)
     if (fm[[2L]] == 1)
@@ -1685,6 +1741,8 @@ handle_assocmod <- function(data, assoc, y_mod, grp_stuff, ids, times,
   form_new <- reformulate(rhs, response = NULL)
   df <- get_all_vars(form_new, data)
   df <- df[complete.cases(df), , drop = FALSE]
+  
+  df$offset <- 0 # force offset to zero for assoc term
 
   # Declare df as a data.table for merging with quadrature points
   dt <- prepare_data_table(df, id_var = id_var, time_var = time_var, 
