@@ -42,7 +42,8 @@
 #'   an \eqn{S} by \eqn{Npat} matrix where \eqn{Npat} is the number of individuals.
 #'   
 #'   
-#' @examples 
+#' @examples
+#' if (.Platform$OS.type != "windows" || .Platform$r_arch != "i386") {
 #' \donttest{
 #'  roaches$roach100 <- roaches$roach1 / 100
 #'  fit <- stan_glm(
@@ -52,7 +53,8 @@
 #'     family = poisson(link = "log"),
 #'     prior = normal(0, 2.5),
 #'     prior_intercept = normal(0, 10),
-#'     iter = 500 # to speed up example
+#'     iter = 500, # just to speed up example,
+#'     refresh = 0
 #'  )
 #'  ll <- log_lik(fit)
 #'  dim(ll)
@@ -66,11 +68,9 @@
 #'  dim(ll2)
 #'  all.equal(ncol(ll2), nrow(nd))
 #' }
-#'
+#' }
 log_lik.stanreg <- function(object, newdata = NULL, offset = NULL, ...) {
-  if (!used.sampling(object))
-    STOP_sampling_only("Pointwise log-likelihood matrix")
-  newdata <- validate_newdata(newdata)
+  newdata <- validate_newdata(object, newdata, m = NULL)
   calling_fun <- as.character(sys.call(-1))[1]
   dots <- list(...)
   if (is.stanmvreg(object)) {
@@ -83,6 +83,7 @@ log_lik.stanreg <- function(object, newdata = NULL, offset = NULL, ...) {
     m <- NULL
   }
   
+  newdata <- validate_newdata(object, newdata = newdata, m = m)
   args <- ll_args.stanreg(object, newdata = newdata, offset = offset, 
                           reloo_or_kfold = calling_fun %in% c("kfold", "reloo"), 
                           ...)
@@ -199,7 +200,9 @@ ll_fun <- function(x, m = NULL) {
 #   just a single MCMC draw).
 # @return a named list with elements data, draws, S (posterior sample size) and
 #   N = number of observations
+#' @noRd
 ll_args <- function(object, ...) UseMethod("ll_args")
+#' @exportS3Method NULL
 ll_args.stanreg <- function(object, newdata = NULL, offset = NULL, m = NULL, 
                             reloo_or_kfold = FALSE, ...) {
   validate_stanreg_object(object)
@@ -235,7 +238,11 @@ ll_args.stanreg <- function(object, newdata = NULL, offset = NULL, m = NULL,
   }
   if (is.stanmvreg(object) && !is.null(dots$stanmat)) {
     stanmat <- dots$stanmat # potentially use a stanmat with a single draw
-  }  
+  }
+  
+  if (!is.null(object$dropped_cols)) {
+    x <- x[, !(colnames(x) %in% object$dropped_cols), drop = FALSE]
+  }
   
   if (!is_polr(object)) { # not polr or scobit model
     fname <- f$family
@@ -319,8 +326,9 @@ ll_args.stanreg <- function(object, newdata = NULL, offset = NULL, m = NULL,
     }
     data <- data.frame(y, x)
     draws$beta <- stanmat[, colnames(x), drop = FALSE]
-    patt <- if (length(unique(y)) == 2L) "(Intercept)" else "|"
-    zetas <- grep(patt, colnames(stanmat), fixed = TRUE, value = TRUE)
+    zetas <- grep(pattern = if (length(unique(y)) == 2L) "(Intercept)" else "|", 
+                  x = colnames(stanmat), 
+                  fixed = TRUE, value = TRUE)
     draws$zeta <- stanmat[, zetas, drop = FALSE]
     draws$max_y <- max(y)
     if ("alpha" %in% colnames(stanmat)) { 
@@ -455,8 +463,8 @@ ll_args.stanreg <- function(object, newdata = NULL, offset = NULL, m = NULL,
 .ll_polr_i <- function(data_i, draws) {
   eta <- linear_predictor(draws$beta, .xdata(data_i), data_i$offset)
   f <- draws$f
-  J <- draws$max_y
   y_i <- data_i$y
+  J <- ncol(draws$zeta) + 1
   linkinv <- polr_linkinv(f)
   if (is.null(draws$alpha)) {
     if (y_i == 1) {
@@ -505,6 +513,7 @@ ll_args.stanreg <- function(object, newdata = NULL, offset = NULL, m = NULL,
 # @param pars Output from extract_pars
 # @param m Integer specifying which submodel
 # @param reloo_or_kfold logical. TRUE if ll_args is for reloo or kfold
+#' @exportS3Method NULL
 ll_args.stanjm <- function(object, data, pars, m = 1, 
                            reloo_or_kfold = FALSE, ...) {
   validate_stanjm_object(object)
@@ -521,7 +530,8 @@ ll_args.stanjm <- function(object, data, pars, m = 1,
     y <- data$y[[m]]
     x <- data$yX[[m]]
     z <- t(data$yZt[[m]])
-    Z_names <- data$yZ_names[[m]]    
+    Z_names <- data$yZ_names[[m]] 
+    offset <- data$yOffset[[m]]
   } else { 
     # for stan_mvmer models, log_lik is only ever called for
     # one submodel at a time, so data is for one submodel
@@ -529,9 +539,14 @@ ll_args.stanjm <- function(object, data, pars, m = 1,
     x <- data$X
     z <- t(data$Zt)
     Z_names <- data$Z_names
+    offset <- data$yOffset
   }
   if (!is.binomial(fname)) {
-    dat <- data.frame(y, x)
+    if (!is.null(offset)) {
+      dat <- data.frame(y, x, offset)
+    } else {
+      dat <- data.frame(y, x)
+    }
   } else {
     if (NCOL(y) == 2L) {
       trials <- rowSums(y)
@@ -542,7 +557,11 @@ ll_args.stanjm <- function(object, data, pars, m = 1,
         y <- fac2bin(y)
       stopifnot(all(y %in% c(0, 1)))
     }
-    dat <- data.frame(y, trials, x)
+    if (!is.null(offset)) {
+      dat <- data.frame(y, trials, x, offset)
+    } else {
+      dat <- data.frame(y, trials, x)
+    }
   }  
   dat <- cbind(dat, as.matrix(z))
   draws$beta <- stanmat[, nms$y[[m]], drop = FALSE]
@@ -733,6 +752,10 @@ ll_args.stanjm <- function(object, data, pars, m = 1,
   # Linear predictor for the survival submodel
   e_eta <- linear_predictor(pars$ebeta, data$eXq) 
   
+  # Scaling parameter for linear predictor
+  assoc_as_list <- apply(assoc, 2L, c)
+  scale_assoc <- validate_scale_assoc(object$scale_assoc, assoc_as_list)
+  
   # Add on contribution from assoc structure
   if (length(pars$abeta)) {
     M <- get_M(object)
@@ -749,12 +772,12 @@ ll_args.stanjm <- function(object, data, pars, m = 1,
     if (one_draw) {
       aXq <- make_assoc_terms(parts = data$assoc_parts, assoc = assoc, 
                               family = family, beta = pars$beta, b = pars$b)
-      e_eta <- e_eta + linear_predictor.default(pars$abeta, aXq)
+      e_eta <- e_eta + scale_assoc * linear_predictor.default(pars$abeta, aXq)
     } else {
       aXq <- make_assoc_terms(parts = data$assoc_parts, assoc = assoc, 
                               family = family, beta = pars$beta, b = pars$b)
       for (k in 1:length(aXq)) {
-        e_eta <- e_eta + sweep(aXq[[k]], 1L, pars$abeta[,k], `*`)
+        e_eta <- e_eta + scale_assoc[k] * sweep(aXq[[k]], 1L, pars$abeta[,k], `*`)
       }
     }    
   }
@@ -843,10 +866,12 @@ evaluate_log_basehaz <- function(times, basehaz, coefs) {
 #   at which the log hazard was evaluated for each individual.
 # @param qwts A vector of unstandardised GK quadrature weights.
 # @return A vector or matrix of log survival probabilities.
+#' @noRd
 evaluate_log_survival <- function(log_haz, qnodes, qwts) {
   UseMethod("evaluate_log_survival")
 }
 
+#' @exportS3Method NULL
 evaluate_log_survival.default <- function(log_haz, qnodes, qwts) {
   # convert log hazard to hazard
   haz <- exp(log_haz)
@@ -859,6 +884,7 @@ evaluate_log_survival.default <- function(log_haz, qnodes, qwts) {
   -cum_haz
 }
 
+#' @exportS3Method NULL
 evaluate_log_survival.matrix <- function(log_haz, qnodes, qwts) {
   # convert log hazard to hazard
   haz <- exp(log_haz)
