@@ -15,6 +15,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+
+#' Logit and inverse logit
+#' 
+#' @export
+#' @param x Numeric vector. 
+#' @return A numeric vector the same length as \code{x}.
+logit <- function(x) stats::qlogis(x)
+
+#' @rdname logit
+#' @export
+invlogit <- function(x) stats::plogis(x)
+
 # Set arguments for sampling 
 #
 # Prepare a list of arguments to use with \code{rstan::sampling} via
@@ -144,7 +156,7 @@ used.variational <- function(x) {
   x$algorithm %in% c("meanfield", "fullrank")
 }
 
-# Test if stanreg object used stan_(g)lmer
+# Test if stanreg object used stan_[gn]lmer
 #
 # @param x A stanreg object.
 is.mer <- function(x) {
@@ -187,6 +199,17 @@ STOP_not_optimizing <- function(what) {
   stop(msg, call. = FALSE)
 }
 
+# Consistent error message to use when something is only available for models
+# fit using MCMC or optimization but not VB
+# 
+# @param what An optional message to prepend to the default message.
+STOP_not_VB <- function(what) {
+  msg <- "not available for models fit using algorithm='meanfield|fullrank'."
+  if (!missing(what)) 
+    msg <- paste(what, msg)
+  stop(msg, call. = FALSE)
+}
+
 # Message to issue when fitting model with ADVI but 'QR=FALSE'. 
 recommend_QR_for_vb <- function() {
   message(
@@ -323,15 +346,32 @@ validate_glm_formula <- function(f) {
 }
 
 
+# Check if model formula has something on the LHS of ~
+# @param f Model formula
+# @return FALSE if there is no outcome on the LHS of the formula
+has_outcome_variable <- function(f) {
+  tt <- terms(as.formula(f))
+  if (attr(tt, "response") == 0) {
+    return(FALSE)
+  } else {
+    return(TRUE)
+  }
+}
+
+
 # Check if any variables in a model frame are constants
-# (the exception is that a constant variable of all 1's is allowed)
+#
+# exceptions: constant variable of all 1's is allowed and outcomes with all 0s
+# or 1s are allowed (e.g., for binomial models)
 # 
 # @param mf A model frame or model matrix
 # @return If no constant variables are found mf is returned, otherwise an error
 #   is thrown.
 check_constant_vars <- function(mf) {
-  # don't check if columns are constant for binomial
-  mf1 <- if (NCOL(mf[, 1]) == 2) mf[, -1, drop=FALSE] else mf
+  mf1 <- mf
+  if (NCOL(mf[, 1]) == 2 || all(mf[, 1] %in% c(0, 1))) {
+    mf1 <- mf[, -1, drop=FALSE] 
+  }
   
   lu1 <- function(x) !all(x == 1) && length(unique(x)) == 1
   nocheck <- c("(weights)", "(offset)", "(Intercept)")
@@ -505,9 +545,11 @@ set_prior_scale <- function(scale, default, link) {
 # @param x Predictor matrix.
 # @param offset Optional offset vector.
 # @return A vector or matrix.
+#' @noRd
 linear_predictor <- function(beta, x, offset = NULL) {
   UseMethod("linear_predictor")
 }
+#' @exportS3Method NULL
 linear_predictor.default <- function(beta, x, offset = NULL) {
   eta <- as.vector(if (NCOL(x) == 1L) x * beta else x %*% beta)
   if (length(offset))
@@ -515,6 +557,7 @@ linear_predictor.default <- function(beta, x, offset = NULL) {
   
   return(eta)
 }
+#' @exportS3Method NULL
 linear_predictor.matrix <- function(beta, x, offset = NULL) {
   if (NCOL(beta) == 1L) 
     beta <- as.matrix(beta)
@@ -553,6 +596,10 @@ get_x.default <- function(object, ...) {
   object[["x"]] %ORifNULL% model.matrix(object)
 }
 #' @export
+get_x.betareg <- function(object, ...) {
+  model.matrix(object$terms[["mean"]], model.frame(object), contrasts = object$contrasts[["contrasts"]])
+}
+#' @export
 get_x.gamm4 <- function(object, ...) {
   as.matrix(object[["x"]])
 }
@@ -590,19 +637,24 @@ get_z.stanmvreg <- function(object, m = NULL, ...) {
 # @param x A stanreg object, family object, or string. 
 # @param ... Other arguments passed to methods. For a \code{stanmvreg} object
 #   this can be an integer \code{m} specifying the submodel.
-# @return The inverse link function associated with x.
+# @return The inverse link function associated with x
+#' @noRd
 linkinv <- function(x, ...) UseMethod("linkinv")
+#' @exportS3Method NULL
 linkinv.stanreg <- function(x, ...) {
   if (is(x, "polr")) polr_linkinv(x) else family(x)$linkinv
 }
+#' @exportS3Method NULL
 linkinv.stanmvreg <- function(x, m = NULL, ...) {
   ret <- lapply(family(x), `[[`, "linkinv")
   stub <- get_stub(x)
   if (!is.null(m)) ret[[m]] else list_nms(ret, stub = stub)
 }
+#' @exportS3Method NULL
 linkinv.family <- function(x, ...) {
   x$linkinv
 }
+#' @exportS3Method NULL
 linkinv.character <- function(x, ...) {
   stopifnot(length(x) == 1)
   polr_linkinv(x)
@@ -659,12 +711,16 @@ check_reTrms <- function(reTrms) {
 }
 
 #' @importFrom lme4 glmerControl
-make_glmerControl <- function(...) {
+# @param ignore_lhs ignore or throw error if LHS of formula is missing? (relevant if prior_PD is TRUE)
+make_glmerControl <- function(..., ignore_lhs = FALSE, ignore_x_scale = FALSE) {
   glmerControl(check.nlev.gtreq.5 = "ignore",
                check.nlev.gtr.1 = "stop",
                check.nobs.vs.rankZ = "ignore",
                check.nobs.vs.nlev = "ignore",
-               check.nobs.vs.nRE = "ignore", ...)  
+               check.nobs.vs.nRE = "ignore", 
+               check.formula.LHS = if (ignore_lhs) "ignore" else "stop",
+               check.scaleX = if (ignore_x_scale) "ignore" else "warning",
+               ...)  
 }
 
 # Check if a fitted model (stanreg object) has weights
@@ -724,6 +780,9 @@ validate_data <- function(data, if_missing = NULL) {
     stop("'data' must be a data frame.", call. = FALSE)
   }
   
+  # drop other classes (e.g. 'tbl_df', 'tbl', 'data.table')
+  data <- as.data.frame(data)
+  
   drop_redundant_dims(data)
 }
 
@@ -740,27 +799,40 @@ warn_data_arg_missing <- function() {
 
 # Validate newdata argument for posterior_predict, log_lik, etc.
 #
-# Doesn't check if the correct variables are included (that's done in pp_data),
-# just that newdata is either NULL or a data frame with no missing values. Also
-# drops any unused dimensions in variables (e.g. a one column matrix inside a
-# data frame is converted to a vector).
-# 
-# @param x User's 'newdata' argument
-# @return Either NULL or a data frame
+# Checks for NAs in used variables only (but returns all variables), 
+# and also drops any unused dimensions in variables (e.g. a one column 
+# matrix inside a data frame is converted to a vector).
 #
-validate_newdata <- function(x) {
-  if (is.null(x)) {
-    return(NULL)
+# @param object stanreg object
+# @param newdata NULL or a data frame
+# @pararm m For stanmvreg objects, the submodel (passed to formula())
+# @return NULL or a data frame
+#
+validate_newdata <- function(object, newdata = NULL, m = NULL) {
+  if (is.null(newdata)) {
+    return(newdata)
   }
-  if (!is.data.frame(x)) {
+  if (!is.data.frame(newdata)) {
     stop("If 'newdata' is specified it must be a data frame.", call. = FALSE)
   }
-  if (any(is.na(x))) {
+  
+  # drop other classes (e.g. 'tbl_df', 'tbl')
+  newdata <- as.data.frame(newdata)
+  if (nrow(newdata) == 0) {
+    stop("If 'newdata' is specified it must have more than 0 rows.", call. = FALSE)
+  }
+  
+  # only check for NAs in used variables
+  vars <- all.vars(formula(object, m = m))
+  newdata_check <- newdata[, colnames(newdata) %in% vars, drop=FALSE]
+  if (any(is.na(newdata_check))) {
     stop("NAs are not allowed in 'newdata'.", call. = FALSE)
   }
   
-  x <- as.data.frame(x)
-  drop_redundant_dims(x)
+  if (ncol(newdata) > 0) {
+    newdata <- drop_redundant_dims(newdata)
+  }
+  return(newdata)
 }
 
 
@@ -1658,15 +1730,6 @@ stop2 <- function(...) {
   stop(..., call. = FALSE)
 }
 
-warning2 <- function(...) {
-  warning(..., call. = FALSE)
-}
-
-SW <- function(expr) {
-  # just a short form for suppressWarnings
-  base::suppressWarnings(expr)
-}
-
 is_null <- function(x) {
   # check if an object is NULL
   is.null(x) || ifelse(is.vector(x), all(sapply(x, is.null)), FALSE)
@@ -1680,17 +1743,3 @@ rm_null <- function(x, recursive = TRUE) {
   }
   x
 }
-
-isFALSE <- function(x) {
-  identical(FALSE, x)
-}
-
-is_equal <- function(x, y, ...) {
-  isTRUE(all.equal(x, y, ...))
-}
-
-is_like_factor <- function(x) {
-  # check if x behaves like a factor in design matrices
-  is.factor(x) || is.character(x) || is.logical(x)
-}
-
