@@ -52,7 +52,7 @@
 #' }
 #' \item{\code{confint}}{
 #' For models fit using optimization, confidence intervals are returned via a 
-#' call to \code{\link[stats]{confint.default}}. If \code{algorithm} is 
+#' call to \code{\link[stats:confint]{confint.default}}. If \code{algorithm} is 
 #' \code{"sampling"}, \code{"meanfield"}, or \code{"fullrank"}, the
 #' \code{confint} will throw an error because the
 #' \code{\link{posterior_interval}} function should be used to compute Bayesian 
@@ -267,14 +267,15 @@ ranef_template <- function(object) {
     new_formula <- as.formula(paste(new_formula_lhs, "~", new_formula_rhs))
   }
   
-  if (stan_fun != "stan_nlmer" && is.gaussian(object$family$family)) {
+  if (stan_fun != "stan_nlmer" && 
+      (is.gaussian(object$family$family) || is.beta(object$family$family))) {
     stan_fun <- "stan_lmer"
   }
   lme4_fun <- switch(
     stan_fun,
     "stan_lmer" = "lmer",
     "stan_nlmer" = "nlmer",
-    "glmer" # for stan_glmer, stan_glmer.nb, stan_gamm4 (unless gaussian)
+    "glmer" # for stan_glmer, stan_glmer.nb, stan_gamm4 (unless gaussian or beta)
   )
   cntrl_args <- list(optimizer = "Nelder_Mead", optCtrl = list(maxfun = 1))
   if (lme4_fun != "nlmer") { # nlmerControl doesn't allow these
@@ -395,7 +396,7 @@ family.stanreg <- function(object, ...) object$family
 #' @keywords internal
 #' @export
 #' @param formula,... See \code{\link[stats]{model.frame}}.
-#' @param fixed.only See \code{\link[lme4]{model.frame.merMod}}.
+#' @param fixed.only See \code{\link[lme4:merMod-class]{model.frame.merMod}}.
 #' 
 model.frame.stanreg <- function(formula, fixed.only = FALSE, ...) {
   if (is.mer(formula)) {
@@ -458,7 +459,7 @@ terms.stanreg <- function(x, ..., fixed.only = TRUE, random.only = FALSE) {
     attr(Terms, "predvars") <- attr(terms(fr), "predvars.fixed")
   } 
   if (random.only) {
-    Terms <- terms.formula(lme4::subbars(formula.stanreg(x, random.only = TRUE)))
+    Terms <- terms.formula(reformulas::subbars(formula.stanreg(x, random.only = TRUE)))
     attr(Terms, "predvars") <- attr(terms(fr), "predvars.random")
   }
   
@@ -512,7 +513,7 @@ coef_mer <- function(object, ...) {
 
 justRE <- function(f, response = FALSE) {
   response <- if (response && length(f) == 3) f[[2]] else NULL
-  reformulate(paste0("(", vapply(lme4::findbars(f), 
+  reformulate(paste0("(", vapply(reformulas::findbars(f), 
                                  function(x) paste(deparse(x, 500L), 
                                                    collapse = " "), 
                                  ""), ")"), 
@@ -532,7 +533,7 @@ formula_mer <- function (x, fixed.only = FALSE, random.only = FALSE, ...) {
   }
   if (fixed.only) {
     form <- attr(fr, "formula")
-    form[[length(form)]] <- lme4::nobars(form[[length(form)]])
+    form[[length(form)]] <- reformulas::nobars(form[[length(form)]])
   }
   if (random.only)
     form <- justRE(form, response = TRUE)
