@@ -594,7 +594,8 @@ test_that("print and summary methods ok for mcmc and vb", {
   expect_output(print(s), "stan_glmer")
   expect_output(
     print(s), 
-    paste(rstanarm:::posterior_sample_size(example_model)), "(posterior sample size)"
+    paste(rstanarm:::posterior_sample_size(example_model), "(posterior sample size)"),
+    fixed = TRUE
   )
   expect_identical(attr(s, "algorithm"), "sampling")
   expect_identical(colnames(s), colnames(d))
@@ -819,3 +820,53 @@ test_that("stanreg_list works", {
   expect_identical(list1$stan_lmer1, stan_lmer1)
   expect_identical(list1$stan_lmer2, stan_lmer2)
 })
+
+
+# posterior pkg draws formats ---------------------------------------------
+test_that("as_draws methods work", {
+  draws <- as_draws_df(stan_lmer1)
+  expect_equal(posterior::variables(draws), colnames(as.matrix(stan_lmer1)))
+  expect_equal(posterior::nvariables(draws), ncol(as.matrix(stan_lmer1)))
+  expect_equal(posterior::ndraws(draws), ITER)
+  expect_equal(posterior::niterations(draws), ITER/CHAINS)
+  expect_equal(posterior::nchains(draws), CHAINS)
+  
+  draws <- as_draws_df(stan_lmer1, pars = "sigma")
+  expect_equal(posterior::variables(draws), "sigma")
+
+  draws <- as_draws_matrix(stan_lmer1)
+  expect_equal(posterior::variables(draws), colnames(as.matrix(stan_lmer1)))
+  expect_equal(posterior::nvariables(draws), ncol(as.matrix(stan_lmer1)))
+  expect_equal(posterior::ndraws(draws), ITER)
+  expect_equal(posterior::niterations(draws), ITER)
+  expect_equal(posterior::nchains(draws), 1)
+  
+  draws <- as_draws_df(stan_glm_vb1)
+  expect_equal(posterior::variables(draws), colnames(as.matrix(stan_glm_vb1)))
+  expect_equal(posterior::nvariables(draws), ncol(as.matrix(stan_glm_vb1)))
+  expect_equal(posterior::ndraws(draws), 1000)
+  expect_equal(posterior::niterations(draws), 1000)
+  expect_equal(posterior::nchains(draws), 1)
+  
+  draws <- as_draws_df(stan_glm_opt1)
+  expect_equal(posterior::variables(draws), colnames(as.matrix(stan_glm_vb1)))
+  expect_equal(posterior::nvariables(draws), ncol(as.matrix(stan_glm_vb1)))
+  expect_equal(posterior::ndraws(draws), 1000)
+  expect_equal(posterior::niterations(draws), 1000)
+  expect_equal(posterior::nchains(draws), 1)
+  
+  expect_equal(
+    as_draws_list(as_draws_array(stan_polr1)),
+    as_draws_list(stan_polr1)
+  )
+  
+  expect_error(
+    as_draws_array(stan_glm_opt1), 
+    "not fit using MCMC"
+  )
+  expect_error(
+    as_draws_array(stan_glm_vb1), 
+    "not fit using MCMC"
+  )
+})
+
