@@ -1,25 +1,25 @@
 # Part of the rstanarm package for estimating model parameters
 # Copyright (C) 2015, 2016, 2017, 2018, 2019 Trustees of Columbia University
-# 
+#
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 3
 # of the License, or (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-Sys.setenv(USE_CXX14 = 1)
+Sys.setenv(USE_CXX17 = 1)
 set.seed(12345)
 
-MODELS_HOME <- "stan_files"
-INCLUDE_DIR <- "include"
+MODELS_HOME <- system.file("stan", package = "rstanarm", mustWork = TRUE)
+INCLUDE_DIR <- system.file("include", package = "rstanarm", mustWork = TRUE)
 
 context("setup")
 test_that("Stan programs are available", {
@@ -29,7 +29,7 @@ test_that("Stan programs are available", {
 library(rstan)
 Sys.unsetenv("R_TESTS")
 # TBB <- system.file("lib", .Platform$r_arch, package = "RcppParallel", mustWork = TRUE)
-# SH  <- system.file(ifelse(.Platform$OS.type == "windows", "libs", "lib"), 
+# SH  <- system.file(ifelse(.Platform$OS.type == "windows", "libs", "lib"),
 #                    .Platform$r_arch, package = "StanHeaders",  mustWork = TRUE)
 # Sys.setenv(LOCAL_LIBS = paste0("-L", shQuote(TBB), " -tbb -tbbmalloc ",
 #                                "-L", shQuote(SH) , " -lStanHeaders"))
@@ -41,7 +41,7 @@ Sys.unsetenv("R_TESTS")
 
 functions <- sapply(dir(MODELS_HOME, pattern = "stan$", full.names = TRUE), function(f) {
   mc <- readLines(f)
-  mc <- grep("^#include", mc, invert = TRUE, value = TRUE)
+  mc <- grep("#include", mc, invert = TRUE, value = TRUE)
   start <- grep("^functions[[:blank:]]*\\{[[:blank:]]*$", mc)
   if (length(start) == 1) {
     end <- grep("^}[[:blank:]]*$", mc)[1]
@@ -50,15 +50,15 @@ functions <- sapply(dir(MODELS_HOME, pattern = "stan$", full.names = TRUE), func
   } else return(as.character(NULL))
 })
 names(functions) <- basename(names(functions))
-functions <- c(unlist(lapply(file.path(MODELS_HOME, "functions", 
+functions <- c(unlist(lapply(file.path(MODELS_HOME, "functions",
                              c("common_functions.stan",
                                "bernoulli_likelihoods.stan",
                                "binomial_likelihoods.stan",
                                "continuous_likelihoods.stan",
-                               "count_likelihoods.stan", 
-                               "SSfunctions.stan")), 
+                               "count_likelihoods.stan",
+                               "SSfunctions.stan")),
                       FUN = readLines)), unlist(functions))
-model_code <- paste(c("functions {", functions, "}"), collapse = "\n")
+model_code <- paste(c("functions {", functions[grep("CODOLS", functions, invert = TRUE)], "}"), collapse = "\n")
 stanc_ret <- stanc(model_code = model_code, model_name = "Stan Functions",
                    allow_undefined = TRUE)
 expose_stan_functions(stanc_ret, rebuild = TRUE, verbose = TRUE)
@@ -74,7 +74,7 @@ test_that("linkinv_bern returns expected results", {
   for (i in 1:length(links)) {
     eta <- -abs(rnorm(N))
     linkinv <- binomial(link = links[i])$linkinv
-    expect_true(all.equal(linkinv(eta), 
+    expect_true(all.equal(linkinv(eta),
                           linkinv_bern(eta, i)), info = links[i])
   }
 })
@@ -88,8 +88,8 @@ test_that("pw_bern and ll_bern_lp return expected results", {
     expect_true(all.equal(ll0, pw_bern(0, eta0, i)), info = links[i])
     ll1 <- dbinom(1, size = 1, prob = linkinv(eta1), log = TRUE)
     expect_true(all.equal(ll1, pw_bern(1, eta1, i)), info = links[i])
-    expect_true(all.equal(sum(ll0, ll1), 
-                          ll_bern_lp(eta0, eta1, i, c(N,N))), 
+    expect_true(all.equal(sum(ll0, ll1),
+                          bern_lpdf(eta0, eta1, i, c(N,N))),
                 info = links[i])
   }
 })
@@ -101,7 +101,7 @@ test_that("linkinv_binom returns expected results", {
   for (i in 1:length(links)) {
     eta <- -abs(rnorm(N))
     linkinv <- binomial(link = links[i])$linkinv
-    expect_true(all.equal(linkinv(eta), 
+    expect_true(all.equal(linkinv(eta),
                           linkinv_binom(eta, i)), info = links[i])
   }
 })
@@ -113,7 +113,7 @@ test_that("pw_binom and ll_binom_lp return expected results", {
     linkinv <- binomial(link = links[i])$linkinv
     ll <- dbinom(y, size = trials, prob = linkinv(eta), log = TRUE)
     expect_true(all.equal(ll,  pw_binom(y, rep(trials, N), eta, i)), info = links[i])
-    expect_true(all.equal(sum(ll), ll_binom_lp(y, rep(trials, N), eta, i), info = links[i]))
+    expect_true(all.equal(sum(ll), binom_lpmf(y, rep(trials, N), eta, i), info = links[i]))
   }
 })
 
@@ -125,7 +125,7 @@ test_that("linkinv_count returns expected results", {
   for (i in 1:length(links)) {
     eta <- abs(rnorm(N))
     linkinv <- poisson(link = links[i])$linkinv
-    expect_true(all.equal(linkinv(eta), 
+    expect_true(all.equal(linkinv(eta),
                           linkinv_count(eta, i)), info = links[i])
   }
 })
@@ -199,7 +199,7 @@ test_that("pw_gamma implies an actual density", {
     foo <- function(y) {
       exp(pw_gamma(y, rep(eta, length(y)), shape, i))
     }
-    expect_true(all.equal(1, integrate(foo, lower = 0, upper = Inf)$value, tol = 1e-5))    
+    expect_true(all.equal(1, integrate(foo, lower = 0, upper = Inf)$value, tol = 1e-5))
   }
 })
 test_that("GammaReg_log returns the expected results", {
@@ -208,12 +208,12 @@ test_that("GammaReg_log returns the expected results", {
     shape <- rexp(1)
     linkinv <- Gamma(link = links[i])$linkinv
     y <- rgamma(N, shape, rate = 1 / linkinv(eta))
-    expect_true(all.equal(sum(dgamma(y, shape = shape, 
+    expect_true(all.equal(sum(dgamma(y, shape = shape,
                                      rate = shape / linkinv(eta), log = TRUE)),
                           GammaReg(y, eta, shape, i, sum(log(y)))), info = links[i])
   }
 })
-  
+
 # Inverse Gaussian GLM
 links <- c(links, "1/mu^2")
 test_that("linkinv_inv_gaussian returns expected results", {
@@ -234,7 +234,7 @@ rinvGauss <- function(n, mu, lambda) {
   return(out)
 }
 dinvGauss <- function(x, mu, lambda, log = FALSE) {
-  out <- 0.5 * log(0.5 * lambda / pi) - 1.5 * log(x) - 
+  out <- 0.5 * log(0.5 * lambda / pi) - 1.5 * log(x) -
          0.5 * lambda / mu^2 * (x - mu)^2 / x
   if (!log) out <- exp(out)
   return(out)
@@ -246,7 +246,7 @@ test_that("pw_inv_gaussian returns expected results", {
     linkinv <- inverse.gaussian(link = links[i])$linkinv
     y <- rinvGauss(N, linkinv(eta), lambda)
     expect_true(all.equal(dinvGauss(y, linkinv(eta), lambda, log = TRUE),
-                          pw_inv_gaussian(y, eta, lambda, i, log(y), sqrt(y))), 
+                          pw_inv_gaussian(y, eta, lambda, i, log(y), sqrt(y))),
                 info = links[i])
   }
 })
@@ -257,7 +257,7 @@ test_that("pw_inv_gaussian implies an actual density", {
     foo <- function(y) {
       exp(pw_inv_gaussian(y, rep(eta, length(y)), lambda, i, log(y), sqrt(y)))
     }
-    expect_true(all.equal(1, integrate(foo, lower = 0, upper = Inf)$value, tol = 1e-4))    
+    expect_true(all.equal(1, integrate(foo, lower = 0, upper = Inf)$value, tol = 1e-4))
   }
 })
 test_that("inv_gaussian returns expected results", {
@@ -267,8 +267,8 @@ test_that("inv_gaussian returns expected results", {
     linkinv <- inverse.gaussian(link = links[i])$linkinv
     y <- rinvGauss(N, linkinv(eta), lambda)
     expect_true(all.equal(sum(dinvGauss(y, linkinv(eta), lambda, log = TRUE)),
-                          inv_gaussian(y, linkinv_inv_gaussian(eta,i), 
-                                       lambda, sum(log(y)), sqrt(y))), 
+                          inv_gaussian(y, linkinv_inv_gaussian(eta,i),
+                                       lambda, sum(log(y)), sqrt(y))),
                 info = links[i])
   }
 })
@@ -287,8 +287,8 @@ test_that("ll_mvn_ols... returns expected results", {
   sigma <- rexp(1)
   SSR <- crossprod(residuals(ols))[1]
   ll <- sum(dnorm(y, intercept + X %*% beta, sigma, log = TRUE))
-  expect_true(all.equal(ll, ll_mvn_ols(c(intercept, beta), b, 
-                                       crossprod(cbind(1, X)), SSR, 
+  expect_true(all.equal(ll, mvn_ols_lpdf(c(intercept, beta), b,
+                                       crossprod(cbind(1, X)), SSR,
                                        sigma, N)))
   decomposition <- qr(X)
   Q <- qr.Q(decomposition)
@@ -296,7 +296,7 @@ test_that("ll_mvn_ols... returns expected results", {
   R_inv <- qr.solve(decomposition, Q)
   b <- R %*% b[-1]
   beta <- R %*% beta
-  expect_true(all.equal(ll, ll_mvn_ols_qr_lp(beta, b, intercept, mean(y), 
+  expect_true(all.equal(ll, mvn_ols_qr_lpdf(beta, b, intercept, mean(y),
                                              SSR, sigma, N)))
 })
 
@@ -321,14 +321,14 @@ test_that("pw_polr returns expected results", {
     zeta <- sort(rnorm(J-1))
     eta <- c(x %*% beta)
     y <- apply(rmultinom(N, 1, prob = rep(1/J, J)) == 1, 2, which)
-    model <- MASS::polr(as.factor(y) ~ x, method = links[i], 
+    model <- MASS::polr(as.factor(y) ~ x, method = links[i],
                         start = c(beta, zeta), control = list(maxit = 0))
     Pr <- fitted(model)
     Pr <- sapply(1:N, FUN = function(i) Pr[i,y[i]])
     log_pr <- pw_polr(y, eta, zeta, i, 1)
     log_Pr <- log(Pr)
     good <- is.finite(log_pr) & is.finite(log_Pr) & log_Pr > -30
-    expect_equal(log_Pr[good], log_pr[good], info = links[i], 
+    expect_equal(log_Pr[good], log_pr[good], info = links[i],
                  tolerance = 1e-6)
   }
 })
@@ -370,13 +370,13 @@ if (require(lme4) && require(HSAUR3)) test_that("the Stan equivalent of lme4's Z
     Lambdati <- group$Lambdat
     Lind <- group$Lind
     theta <- group$theta
-    
-    group <- rstanarm:::pad_reTrms(Ztlist = group$Ztlist, cnms = group$cnms, 
+
+    group <- rstanarm:::pad_reTrms(Ztlist = group$Ztlist, cnms = group$cnms,
                                    flist = group$flist)
     Z <- group$Z
     p <- sapply(group$cnms, FUN = length)
     l <- sapply(attr(group$flist, "assign"), function(i) nlevels(group$flist[[i]]))
-    
+
     len_theta_L <- sum(choose(p,2), p)
     expect_true(len_theta_L == length(theta))
     dispersion <- runif(1)
@@ -385,24 +385,24 @@ if (require(lme4) && require(HSAUR3)) test_that("the Stan equivalent of lme4's Z
     zeta <- as.array(rgamma(sum(p[p > 1]), shape = 1, scale = 1))
     rho <- as.array(rbeta(sum(p - 1), 1, 1))
     z_T <- as.array(rnorm(sum(pmax(0, choose(p,2) - 1))))
-    
+
     theta_L <- make_theta_L(len_theta_L, p, dispersion, tau, scale, zeta, rho, z_T)
     expect_true(all(theta_L[theta == 1] > 0))
     Lambdati@x <- theta_L[Lind]
-    
+
     z_b <- rnorm(ncol(Z))
     b <- make_b(z_b, theta_L, p, l)
     mark <- colnames(Z) == ""
-    expect_equal(b[!mark], as.vector(Matrix::t(Lambdati) %*% z_b[!mark]), 
+    expect_equal(b[!mark], as.vector(Matrix::t(Lambdati) %*% z_b[!mark]),
                  tol = 1e-14)
-    
+
     parts <- extract_sparse_parts(Z)
     Zb <- Z %*% b
     if (all(sapply(group$cnms, FUN = function(x) {
         length(x) == 1 && x == "(Intercept)"
       })) ) {
       V <- matrix(parts$v, nrow = sum(p), ncol = nrow(Z))
-      expect_true(all(V == 
+      expect_true(all(V ==
                       t(as.matrix(as.data.frame(make_V(nrow(Z), nrow(V), parts$v))))))
       expect_equal(Zb@x, apply(V, 2, FUN = function(v) sum(b[v])))
     }
@@ -419,8 +419,8 @@ if (require(lme4) && require(HSAUR3)) test_that("the Stan equivalent of lme4's Z
   test_lme4(glFormula(outcome ~ visit + treatment + (visit|treatment) + (1|patientID),
                       data=toenail, family = binomial)$reTrms)
   data(clouds, package = "HSAUR3")
-  test_lme4(glFormula(rainfall ~ sne + cloudcover + prewetness + echomotion + 
-                        (1 + sne + cloudcover + prewetness|seeding) +  
+  test_lme4(glFormula(rainfall ~ sne + cloudcover + prewetness + echomotion +
+                        (1 + sne + cloudcover + prewetness|seeding) +
                         (1 + sne + cloudcover + prewetness||echomotion),
                       data=clouds, family = gaussian)$reTrms)
   test_lme4(glFormula(angle ~ recipe + temp + (1|recipe:replicate), data = cake)$reTrms)
@@ -439,7 +439,7 @@ test_that("SSasymp works", {
   Lob.329 <- Loblolly[ Loblolly$Seed == "329", ]
   Asym <- 100
   resp0 <- -8.5
-  lrc <- -3.2  
+  lrc <- -3.2
   Phi <- cbind(Asym, resp0, lrc)
   expect_true(all.equal(SSasymp( Lob.329$age, Asym, resp0, lrc ),
                         SS_asymp( Lob.329$age, Phi ), check.attributes = FALSE))
@@ -570,7 +570,7 @@ test_that("linkinv_beta returns expected results", {
   for (i in 1:length(links)) {
     eta <- -abs(rnorm(N))
     linkinv <- binomial(link = links[i])$linkinv
-    expect_true(all.equal(linkinv(eta), 
+    expect_true(all.equal(linkinv(eta),
                           linkinv_beta(eta, i)), info = links[i])
   }
 })
@@ -596,7 +596,7 @@ test_that("ll_clogit_lp (which calls log_clogit_denom) returns the expected resu
   y <- infert$case == 1
   s <- aggregate(y, by = list(infert$stratum), FUN = sum)$x
   obs <- aggregate(y, by = list(infert$stratum), FUN = length)$x
-  ll <- ll_clogit_lp(eta0 = eta[!y], eta1 = eta[y], 
+  ll <- clogit_lpdf(eta0 = eta[!y], eta1 = eta[y],
                      successes = s, failures = obs - s, observations = obs)
   expect_equal(-64.202236924431, ll)
 })
